BEGIN
  FOR k IN (SELECT null FROM all_synonyms WHERE owner = 'PUBLIC' and synonym_name = 'REVIEWER_ITEM_ACTIONS_V') LOOP
    EXECUTE IMMEDIATE 'DROP PUBLIC SYNONYM REVIEWER_ITEM_ACTIONS_V';
  END LOOP;
  FOR i IN (SELECT null FROM all_views WHERE owner = user and view_name = 'REVIEWER_ITEM_ACTIONS_V') LOOP
    EXECUTE IMMEDIATE 'DROP VIEW REVIEWER_ITEM_ACTIONS_V';
  END LOOP;
    FOR k IN (SELECT null FROM all_synonyms WHERE owner = 'PUBLIC' and synonym_name = 'SUSER_V') LOOP
    EXECUTE IMMEDIATE 'DROP PUBLIC SYNONYM SUSER_V';
  END LOOP;
  FOR i IN (SELECT null FROM all_views WHERE owner = user and view_name = 'SUSER_V') LOOP
    EXECUTE IMMEDIATE 'DROP VIEW SUSER_V';
  END LOOP;
END;
/

CREATE VIEW reviewer_item_actions_v AS
  SELECT 
    review_instance.id,
    review_instance.status review_status,
    review_instance.start_date,
    review_instance.end_date,
    review_instance.rev_name,
    review_instance.review_type,
    review_instance.unique_review_def_id,
    all_reviewers.reviewer_id,
    all_reviewers.display_name reviewer_name,
    all_reviewers.item_id,
    all_reviewers.item_name,
    all_reviewers.item_type,
    all_reviewers.reviewer_comment,
    all_reviewers.action,
    all_reviewers.reviewer_type,
    all_reviewers.review_order
  FROM (SELECT 
      rev_inst.id,
      rev_inst.status,
      rev_inst.start_date,
      rev_inst.end_date,
      COALESCE(staged_defn.name, defn.name) rev_name,
      defn.review_type,
      defn.unique_review_def_id
    FROM review_instance rev_inst
      JOIN review_definition defn ON defn.id = rev_inst.review_definition
      LEFT JOIN review_definition staged_defn ON staged_defn.id = rev_inst.staged_review_definition_id
    WHERE rev_inst.status NOT LIKE 'PREVIEW%') review_instance
    JOIN (SELECT 
        item.review_instance_id,
        COALESCE(ria.acted_by_user_unique_id, ria.reassigned_by_user_unique_id) reviewer_id,
        COALESCE(usr.display_name, usr2.display_name) display_name,
        TO_CHAR(ria.action) action,
        ria.review_order,
        item.item_type,
        item.id item_id,
        COALESCE(perm.name, acct.name, trole.name, brole.name) item_name,
        ria.reviewer_type,
        ria.reviewer_comment
      FROM review_item item
        JOIN review_item_action ria ON ria.review_item_id = item.id AND ria.action IN ('KEEP', 'REMOVE', 'MODIFY_ACCESS', 'REASSIGNED')
        LEFT JOIN spermission perm ON perm.unique_permission_id = item.unique_permission_id AND perm.effective = 1
        LEFT JOIN saccount acct ON acct.unique_account_id = item.unique_account_id AND acct.effective = 1
        LEFT JOIN role_policy trole ON trole.id = item.role_id AND trole.state = 'ACTIVE'
        LEFT JOIN business_role brole ON brole.id = item.business_role_id AND brole.approval_state != 'ARCHIVE'
        LEFT JOIN suser usr ON usr.unique_user_id = ria.acted_by_user_unique_id AND usr.effective = 1
        LEFT JOIN suser usr2 ON usr2.unique_user_id = ria.reassigned_by_user_unique_id AND usr2.effective = 1

      UNION ALL

      SELECT
        auditor.review_instance_id,
        rui.unique_user_id auditor_unique_id,
        usr.display_name,
          CASE
            WHEN rev_inst.status = 'CERTIFIED' THEN 'ACCEPTED'
            ELSE 'REJECTED'
          END AS rev_action,
        auditor.review_order,
        NULL item_type,
        NULL item_id,
        NULL item_name,
        auditor.type,
        rev_inst.auditor_comment
      FROM reviewer_instance auditor
        JOIN reviewer_user_instance rui ON auditor.id = rui.reviewer_instance_id
        JOIN review_instance rev_inst ON auditor.review_instance_id = rev_inst.id AND rev_inst.status IN ('CERTIFIED', 'NOT_CERTIFIED')
        JOIN suser usr ON usr.unique_user_id = rui.unique_user_id AND usr.effective = 1
      WHERE auditor.type = 'AUDITOR') all_reviewers ON all_reviewers.review_instance_id = review_instance.id AND all_reviewers.reviewer_id IS NOT NULL;

CREATE VIEW suser_v AS
  SELECT
    suser.id,
    suser.deleted,
    suser.department,
    suser.display_description,
    suser.display_name,
    suser.dn,
    suser.effective,
    suser.employee_status,
    suser.employee_type,
    TO_CHAR(suser.first_name) first_name,
    suser.full_name,
    suser.guid,
    suser.home_phone,
    suser.idm_dn,
    suser.idm_tree_name,
    suser.is_curated,
    suser.job_code,
    TO_CHAR(suser.job_title) job_title,
    TO_CHAR(suser.last_name) last_name,
    suser.location,
    suser.merged,
    TO_CHAR(suser.middle_name) middle_name,
    suser.office_phone,
    suser.other_phone,
    suser.preferred_locale,
    suser.provisioning_id,
    suser.secondary_supervisor_id,
    suser.supervisor_attr_key,
    suser.supervisor_id,
    suser.supervisor_unique_id,
    TO_CHAR(suser.unique_user_id) unique_user_id,
    TO_CHAR(suser.user_id) user_id,
    suser.business_unit,
    suser.collected_entity,
    suser.effective_entity,
    suser.identity_source,
    suser.sversion,
    TO_CHAR(suser.workforce_id) workforce_id,
    suser_email.email,
    suser.risk
  FROM suser
    LEFT JOIN (SELECT
        suser.id suser,
        suser.display_name,
        LISTAGG(TO_CHAR(suser_email.email), ' | ') WITHIN GROUP(ORDER BY email) email
      FROM suser
        JOIN suser_email ON suser.id = suser_email.suser
      WHERE suser.effective = 1 AND suser.deleted = 0  
      GROUP BY suser.id, suser.display_name) suser_email ON suser.id = suser_email.suser
  WHERE suser.deleted = 0 AND suser.effective = 1;

GRANT SELECT ON igops.reviewer_item_actions_v TO ig_report_role;
CREATE PUBLIC SYNONYM reviewer_item_actions_v FOR igops.reviewer_item_actions_v;

GRANT SELECT ON igops.suser_v TO ig_report_role;
CREATE PUBLIC SYNONYM suser_v FOR igops.suser_v;